/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.internal;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.HeaderParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

import org.ow2.bonita.facade.def.element.BusinessArchive;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.DeploymentException;
import org.ow2.bonita.facade.exception.PrivilegeNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.RuleAlreadyExistsException;
import org.ow2.bonita.facade.exception.RuleNotFoundException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.exception.UndeletableProcessException;
import org.ow2.bonita.facade.exception.UserNotFoundException;
import org.ow2.bonita.facade.privilege.PrivilegePolicy;
import org.ow2.bonita.facade.privilege.Rule;
import org.ow2.bonita.facade.privilege.Rule.RuleType;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.util.BusinessArchiveFactory;

/**
 * 
 * @author Elias Ricken de Medeiros
 *
 */
@Path("/API/managementAPI/")
@Produces({"text/*","application/xml"})
@Consumes({"application/x-www-form-urlencoded","text/*", "application/xml"})
public interface AbstractRemoteManagementAPI extends Remote {

	/**
	 * Deploys the given businessArchive into Bonita server.
   * A businessArchive can be build using {@link BusinessArchiveFactory} class.
   * @param businessArchive businessArchive to deploy
	 * @param options the options map (domain, queyList, user)
	 * @return the deployed process
	 * @throws DeploymentException
	 * @throws RemoteException
	 */
  @Consumes({"application/octet-stream","text/*", "application/xml"})
	@POST @Path("deploy")  
  ProcessDefinition deploy(
  		final BusinessArchive businessArchive, 
  		@HeaderParam("options") final Map<String, String> options) 
	throws DeploymentException, RemoteException;

	/**
	 * Deploys a JAR giving its bytes table.
   * @param jarName the jar name
   * @param jar the bytes table of the JAR.
	 * @param options the options map (domain, queyList, user)
	 * @throws DeploymentException
	 * @throws RemoteException
	 */
  @Consumes({"application/octet-stream", "text/*", "application/xml"})
	@POST @Path("deployJar/{jarName}")
  void deployJar(
  		@PathParam("jarName")String jarName, 
  		byte[] jar, 
  		@HeaderParam("options") final Map<String, String> options) 
	throws DeploymentException, RemoteException;

	/**
	 * Removes a JAR according to its name.
   * @param jarName the JAR name.
	 * @param options the options map (domain, queyList, user)
	 * @throws DeploymentException
	 * @throws RemoteException
	 */
	@POST @Path("removeJar/{jarName}")
  void removeJar(
  		@PathParam("jarName") String jarName, 
  		@FormParam("options") final Map<String, String> options) 
	throws DeploymentException, RemoteException;

	/**
	 * Deletes from journal and history:<br>
   * <ul>
   *   <li>the process</li>
   *   <li>all instances of this process.</li>
   * <ul>
   *
   * @param processUUID the process UUID to delete
	 * @param options the options map (domain, queyList, user)
	 * @throws ProcessNotFoundException
	 * @throws UndeletableProcessException
	 * @throws UndeletableInstanceException
	 * @throws RemoteException
	 */
	@POST @Path("deleteProcess/{processUUID}")
  void deleteProcess(
  		@PathParam("processUUID") ProcessDefinitionUUID processUUID, 
  		@FormParam("options") final Map<String, String> options) 
	throws ProcessNotFoundException, UndeletableProcessException, UndeletableInstanceException, RemoteException;

	/**
	 * Deletes from journal and history :<br>
   * <ul>
   *   <li>all processes</li>
   *   <li>all instances of these processes.</li>
   * <ul>
	 * @param options the options map (domain, queyList, user)
	 * @throws UndeletableProcessException
	 * @throws UndeletableInstanceException
	 * @throws RemoteException
	 */
	@POST @Path("deleteAllProcesses")
  void deleteAllProcesses(
  		@FormParam("options") final Map<String, String> options) 
	throws UndeletableProcessException, UndeletableInstanceException, RemoteException;

	/**
	 * Gets the current logged user. 
	 * @param options the options map (domain, queyList, user)
	 * @return the logged user
	 * @throws RemoteException
	 */
	@POST @Path("getLoggedUser")
  String getLoggedUser(
  		@FormParam("options") final Map<String, String> options) 
	throws RemoteException;

	/**
	 * Gets the name of all available JARs.
	 * @param options the options map (domain, queyList, user)
	 * @return a collection of JAR names
	 * @throws RemoteException
	 */
	@POST @Path("getAvailableJars")
  Set<String> getAvailableJars(
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
	/**
	 * Adds or updates a meta data.
   * @param key the meta data key
   * @param value the meta data value
	 * @param options the options map (domain, queyList, user)
	 * @throws RemoteException
	 */
	@POST @Path("addMetaData/{key}/{value}")
  void addMetaData(
  		@PathParam("key") final String key, 
  		@PathParam("value") final String value, 
  		@FormParam("options") final Map<String, String> options) 
	throws RemoteException;

	/**
	 * Obtains a meta data.
   * @param key the key of the meta data
	 * @param options the options map (domain, queyList, user)
	 * @return the value of the meta data
	 * @throws RemoteException
	 */
	@POST @Path("getMetaData/{key}")
  String getMetaData(
  		@PathParam("key") final String key, 
  		@FormParam("options") final Map<String, String> options) 
	throws RemoteException;

	/**
	 * Deletes a meta data.
   * @param key the key of the meta data
	 * @param options the options map (domain, queyList, user)
	 * @throws RemoteException
	 */
	@POST @Path("deleteMetaData/{key}")
  void deleteMetaData(
  		@PathParam("key") final String key, 
  		@FormParam("options") final Map<String, String> options) 
	throws RemoteException;

	/**
	 * Disables a process.
   * @param processUUID the process definition UUID
	 * @param options the options map (domain, queyList, user)
	 * @throws DeploymentException
	 * @throws RemoteException
	 */
	@POST @Path("disable/{processUUID}")
  void disable(
  		@PathParam("processUUID") final ProcessDefinitionUUID processUUID, 
  		@FormParam("options") final Map<String, String> options) 
	throws DeploymentException, RemoteException;

	/**
	 * Enable a process.
   * @param processUUID the process definition UUID
	 * @param options the options map (domain, queyList, user)
	 * @throws DeploymentException
	 * @throws RemoteException
	 */
	@POST @Path("enable/{processUUID}")
  void enable(
  		@PathParam("processUUID") final ProcessDefinitionUUID processUUID, 
  		@FormParam("options") final Map<String, String> options) 
	throws DeploymentException, RemoteException;

	/**
	 * Archive a process. An archived process cannot be enable anymore.
   * @param processUUID the process definition UUID
	 * @param options the options map (domain, queyList, user)
	 * @throws DeploymentException
	 * @throws RemoteException
	 */
	@POST @Path("archive/{processUUID}")
  void archive(
  		@PathParam("processUUID") final ProcessDefinitionUUID processUUID, 
  		@FormParam("options") final Map<String, String> options) 
	throws DeploymentException, RemoteException;
	
	/**
	 * Check whether a user has administrator privileges
   * @param username the user's user name
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws UserNotFoundException
	 * @throws RemoteException
	 */
	@POST @Path("isUserAdmin/{username}")
  @Produces("text/*")
  Boolean isUserAdmin(
  		@PathParam("username") final String username, 
  		@FormParam("options") final Map<String, String> options) 
	throws UserNotFoundException, RemoteException;
  
	/**
	 * Check some user's credentials
   * @param username the user's user name
   * @param password the user's password
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST @Path("checkUserCredentials/{username}")
  @Produces("text/*")  
  Boolean checkUserCredentials(
  		@PathParam("username") final String username, 
  		@FormParam("password") final String password, 
  		@FormParam("options") final Map<String, String> options) 
	throws RemoteException;

	/**
	  Check some user's credentials
   * @param username the user's user name
   * @param passwordHash the user's password hash
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST
  @Path("checkUserCredentialsWithPasswordHash/{username}")
  @Produces("text/*")  
  Boolean checkUserCredentialsWithPasswordHash(
  		@PathParam("username") final String username, 
  		@FormParam("passwordHash") final String passwordHash, 
  		@FormParam("options") final Map<String, String> options) 
	throws RemoteException;
  
	
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * This method deletes a rule identified by its name.<br>
   * It means that the entities bound to this rule will then follow the global policy (unless they are bound to another rule).
   * @param name
   * @param options the options map (domain, queyList, user)
   * @throws RuleNotFoundException
   * @throws RemoteException
   * @deprecated
   */
  @Deprecated
  @POST @Path("deleteRule/{name}")
  void deleteRule(
  		@PathParam("name") String name, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RemoteException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * Update a rule identified by its name with the values stored in the rule object.
   * @param name
   * @param newRule
   * @param options the options map (domain, queyList, user)
   * @throws RuleNotFoundException
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("updateRule/{name}")
  @Deprecated
  void updateRule(
  		@PathParam("name") String name, 
  		@FormParam("newRule") Rule newRule, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RemoteException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * Get a rule identified by its name.
   * @param name
   * @param options the options map (domain, queyList, user)
   * @return @return a {@link Rule}
   * @throws RuleNotFoundException
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("getRule/{name}")
  @Deprecated
  Rule getRule(
  		@PathParam("name") String name, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RemoteException;
  
  /**
   * Lists all available rules.
   * @param options the options map (domain, queyList, user)
   * @return a set of rules
   * @throws RemoteException
   */
  @POST @Path("getRules")
  @Deprecated
  Set<Rule> getRules(
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * Gets rules from their names.
   * @param ruleNames a set of rule names
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   * @throws RuleNotFoundException
   * @deprecated
   */
  @POST @Path("getRulesFromRuleNames")
  @Deprecated
  Set<Rule> getRules(
  		@FormParam("ruleNames") Set<String> ruleNames, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException, RuleNotFoundException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * This method is the way to impose conditions to the general policy for the given entity.<br>
   * For instance applying rules to a user (entity) defines what access the user has or does not have.<br>
   * @param entityID the entity ID (ex: username...)
   * @param rules rule names
   * @param options the options map (domain, queyList, user)
   * @throws RuleNotFoundException
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("applyRulesToEntity/{entityID}")
  @Deprecated
  void applyRulesToEntity(
  		@PathParam("entityID") String entityID, 
  		@FormParam("rules") Set<String> rules, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RemoteException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * This method is the way to remove conditions to the general policy for the given entity.<br>
   * For instance removing rules previously applied to a user (entity) defines what access the user has or does not have.<br>
   * @param entityID
   * @param rules rule namesID
   * @param rules
   * @param options the options map (domain, queyList, user)
   * @throws RuleNotFoundException
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("removeRulesFromEntity/{entityID}")
  @Deprecated
  void removeRulesFromEntity(
  		@PathParam("entityID") String entityID, 
  		@FormParam("rules") Set<String> rules, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RemoteException;
  
  /**
   * Provides entities with the same privilege than entity.
   * @param entityID
   * @param entities
   * @param options the options map (domain, queyList, user)
   * @throws PrivilegeNotFoundException
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("applySameRulesToOtherEntities/{entityID}")
  @Deprecated
  void applySameRulesToOtherEntities(
  		@PathParam("entityID") String entityID, 
  		@FormParam("entities")Set<String> entities, 
  		@FormParam("options") final Map<String, String> options) 
  throws PrivilegeNotFoundException, RemoteException;
  
  /**
   * @param entityID
   * @param ruleType
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("getApplicableRules/{entityID}/{ruleType}")
  @Deprecated
  Set<Rule> getApplicableRules(
  		@PathParam("entityID") String entityID, 
  		@PathParam("ruleType")RuleType ruleType, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * @param entityID
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("getAllApplicableRulesByEntityID/{entityID}")
  @Deprecated
  Set<Rule> getAllApplicableRules(
  		@PathParam("entityID") String entityID, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * Get the policy for RuleType.PROCESS_START<br>
   * PrivilegePolicy.ALLOW_BY_DEFAULT: Means that by default all entities are allowed to act on everything. Except the items listed in rules.<br>
   * PrivilegePolicy.DENY_BY_DEFAULT: Means that by default all entities are NOT allowed to act on anything. Except the items explicitly listed in rules.<br> 
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("getDefaultPolicy")
  @Deprecated
  PrivilegePolicy getDefaultPolicy(
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
    Update the policy for RuleType.PROCESS_START<br>
   * @param newPolicy the {@link PrivilegePolicy} to apply
   * PrivilegePolicy.ALLOW_BY_DEFAULT: Means that by default all entities are allowed to act on everything. Except the items listed in rules.<br>
   * PrivilegePolicy.DENY_BY_DEFAULT: Means that by default all entities are NOT allowed to act on anything. Except the items explicitly listed in rules.<br>
   * @param negateRules if set to true all the rules will be updated to respect the new policy. If set to false the rules will not be updated.
   * @param options the options map (domain, queyList, user)
   * @throws RemoteException
   * @deprecated
   */
  @POST @Path("setDefaultPolicy/{newPolicy}/{negateRules}")
  @Deprecated
  void setDefaultPolicy(
  		@PathParam("newPolicy")PrivilegePolicy newPolicy, 
  		@PathParam("negateRules")boolean negateRules, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * This method creates a rule identified by its name.<br>
   * It means that the entities bound to this rule will NOT follow the global policy anymore.
   * @param name
   * @param label
   * @param decription
   * @param type
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RuleAlreadyExistsException
   * @throws RemoteException
   */
  @POST @Path("createRule/{name}/{label}/{type}")
  Rule createRule(
  		@PathParam("name") final String name, 
  		@PathParam("label") final String label, 
  		@FormParam("decription") final String decription, 
  		@PathParam("type") final RuleType type, 
  		@FormParam("options") final Map<String, String> options) 
  throws  RuleAlreadyExistsException, RemoteException;
  
  /**
   * Lists the rules according to its type from a specific index.
   * @param ruleType
   * @param fromIndex
   * @param pageSige
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   */
  @POST @Path("getRules/{ruleType}")
  List<Rule> getRules(
  		@PathParam("ruleType")  final RuleType ruleType, 
  		@QueryParam("fromIndex")final int fromIndex, 
  		@QueryParam("pageSige")final int pageSige, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * Get the number of rules having the given type.
   * @param ruleType
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   */
  @POST @Path("getNumberOfRules/{ruleType}")
  long getNumberOfRules(
  		@PathParam("ruleType")  final RuleType ruleType, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * Get a rule type policy.<br>
   * PrivilegePolicy.ALLOW_BY_DEFAULT: Means that by default all entities are allowed to act on everything. Except the items listed in rules.<br>
   * PrivilegePolicy.DENY_BY_DEFAULT: Means that by default all entities are NOT allowed to act on anything. Except the items explicitly listed in rules.<br>
   * @param ruleType the rule type
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   */
  @POST @Path("getRuleTypePolicy/{ruleType}")
  PrivilegePolicy getRuleTypePolicy(
  		@PathParam("ruleType") final RuleType ruleType, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * Update a rule type policy.<br>
   * @param newPolicy the {@link PrivilegePolicy} to apply to the rule type
   * PrivilegePolicy.ALLOW_BY_DEFAULT: Means that by default all entities are allowed to act on everything. Except the items listed in rules.<br>
   * PrivilegePolicy.DENY_BY_DEFAULT: Means that by default all entities are NOT allowed to act on anything. Except the items explicitly listed in rules.<br>
   * @param ruleType the rule type
   * @param options the options map (domain, queyList, user)
   * @throws RemoteException
   */
  @POST @Path("setRuleTypePolicy/{ruleType}/{newPolicy}")
  void setRuleTypePolicy(
  		@PathParam("ruleType")  final RuleType ruleType, 
  		@PathParam("newPolicy")final PrivilegePolicy newPolicy, 
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * This method deletes a rule identified by its name.<br>
   * It means that the entities bound to this rule will then follow the global policy (unless they are bound to another rule).
   * @param ruleUUID
   * @param options the options map (domain, queyList, user)
   * @throws RuleNotFoundException
   * @throws RemoteException
   */
  @POST @Path("deleteRuleByUUID/{ruleUUID}")
  void deleteRuleByUUID(
  		@PathParam("ruleUUID")final String ruleUUID, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RemoteException;
  
  /**
   * Lists all available rules
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RemoteException
   */
  @POST @Path("getAllRules")
  List<Rule> getAllRules(
  		@FormParam("options") final Map<String, String> options) 
  throws RemoteException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * Get a rule identified by its name.
   * @param ruleUUID
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RuleNotFoundException
   * @throws RemoteException
   */
  @POST @Path("getRuleByUUID/{ruleUUID}")
  Rule getRuleByUUID(
  		@PathParam("ruleUUID") final String ruleUUID, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RemoteException;
  
  /**
   * The global policy is applied to all entities.<br>
   * Rules apply conditions to the policy as they are applied to specific entities.<br>
   * Update a rule identified by its name with the values stored in the rule object.
   * @param ruleUUID
   * @param name
   * @param label
   * @param description
   * @param options the options map (domain, queyList, user)
   * @return
   * @throws RuleNotFoundException
   * @throws RuleAlreadyExistsException
   * @throws RemoteException
   */
  @POST @Path("updateRuleByUUID/{ruleUUID}/{name}/{label}")
  Rule updateRuleByUUID(
  		@PathParam("ruleUUID") final String ruleUUID, 
  		@PathParam("name") final String name, 
  		@PathParam("label") final String label, 
  		@FormParam("description")final String description, 
  		@FormParam("options") final Map<String, String> options) 
  throws RuleNotFoundException, RuleAlreadyExistsException, RemoteException;
}
