/**
 * Copyright (C) 2009-2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.internal;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

/**
 * This class provides a brief overview of Bonita Business Activity Monitoring.<br>
 * Note: Using this API can affect engine performances.
 * @author Matthieu Chaffotte, Elias Ricken de Medeiros
 *
 */
@Path("/API/BAMAPI/")
@Produces({"text/*","application/xml" })
@Consumes({"application/x-www-form-urlencoded","text/*", "application/xml" })
public interface RemoteBAMAPI extends Remote {

	/**
	 * Gets the total number of overdue steps (i.e. tasks) of every users.
   * An overdue task is a task which its ended date is exceeded.
	 * @param options the options map (domain, queyList, user)
	 * @return he number of overdue steps.
	 * @throws RemoteException
	 */
	@POST	@Path("getNumberOfOverdueSteps")
  int getNumberOfOverdueSteps(
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;
  
	/**
	 * Gets the total number of steps at risk (i.e. tasks) of every users. A step at risk is
   * a step which its ended date is close to exceed according to the remaining days.
   * @param remainingDays the remaining days before the ended step date is exceeded.
	 * @param options the options map (domain, queyList, user)
	 * @return the number of steps at risk of every users.
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfStepsAtRisk")
  int getNumberOfStepsAtRisk(
      @QueryParam("remainingDays") int remainingDays,
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;
  
	/**
	 * Gets the number of open steps of the logged user.
	 * @param options the options map (domain, queyList, user)
	 * @return the number of open steps of the logged user
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfOpenSteps")
  int getNumberOfOpenSteps(
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;

	/**
	 * Gets the number of overdue steps (i.e. tasks) of the logged user.
   * An overdue task is a task which its ended date is exceeded.
	 * @param options the options map (domain, queyList, user)
	 * @return the number of overdue steps of the logged user.
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfUserOverdueSteps")
  int getNumberOfUserOverdueSteps(
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;
  
	/**
	 * Gets the number of steps at risk (i.e. tasks) of the logged user.
   * A step at risk is a step which its ended date is close to exceed according to the remaining days.
   * @param remainingDays the remaining days before the ended step date is exceeded.
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfUserStepsAtRisk")
  int getNumberOfUserStepsAtRisk(
      @QueryParam("remainingDays") int remainingDays,
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;
  
	/**
	 * Gets the number of open steps of the logged user.
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfUserOpenSteps")
  int getNumberOfUserOpenSteps(
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;

	/**
	 * Gets the list of the number of the finished cases (process instances) per day of every users.
   * @param since the starting date
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST	@Path("getNumberOfFinishedCasesPerDay")
  List<Integer> getNumberOfFinishedCasesPerDay(
      @FormParam("since") Date since,
      @FormParam("options") final Map<String, String> options)
  throws RemoteException;

	/**
	 * Gets the list of the number of executing cases (process instances) per day of every users.
   * @param since the starting date
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST	@Path("getNumberOfExecutingCasesPerDay")
  List<Integer> getNumberOfExecutingCasesPerDay(
      @FormParam("since") Date since,
      @FormParam("options") final Map<String, String> options)
  throws RemoteException;

	/**
	 * Gets the number of open steps of every users for every days since the given day.
   * @param since the day 
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfOpenStepsPerDay")
  List<Integer> getNumberOfOpenStepsPerDay(
      @FormParam("since") Date since,
      @FormParam("options") final Map<String, String> options)
  throws RemoteException;

	/**
	 * Gets the total number of open steps (i.e. tasks) of every users according to the step priority.
   * @param priority the step priority
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfUserOpenStepsByPriority")
  int getNumberOfUserOpenSteps(
      @QueryParam("priority") int priority,
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;

	/**
	 * Gets the total number of finished steps of the logged user since the given date according
   * to the step priority.
   * @param priority the step priority
	 * @param since
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfUserFinishedSteps")
  int getNumberOfUserFinishedSteps(
      @QueryParam("priority") int priority,
      @FormParam("since") Date since, 
  		@FormParam("options") final Map<String, String> options)
	throws RemoteException;

	/**
	 * Gets the total number of finished steps of every users since the given date
   * according to the step priority.
   * @param priority the step priority
	 * @param since
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST @Path("getNumberOfFinishedSteps")
  int getNumberOfFinishedSteps(
      @QueryParam("priority") int priority,
      @FormParam("since") Date since, 
  		@FormParam("options") final Map<String, String> options)
	throws RemoteException;

	/**
	 * Gets the total number of open steps (i.e. tasks) of every users according to the step priority.
   * @param priority the step priority
	 * @param options the options map (domain, queyList, user)
	 * @return
	 * @throws RemoteException
	 */
	@POST	@Path("getNumberOfOpenStepsByPriority")
  int getNumberOfOpenSteps(
      @QueryParam("priority") int priority,
      @FormParam("options") final Map<String, String> options)
	throws RemoteException;

}
